@@ -70,6 +70,7 @@ type Allocation struct {
 	reservedNodeID        string
 	result                AllocationResult
 	releases              []*Allocation
+	preempted             bool
 
 	sync.RWMutex
 }
@@ -356,6 +357,20 @@ func (a *Allocation) GetAllocatedResource() *resources.Resource {
 	return a.allocatedResource
 }
 
+// MarkPreempted marks the allocation as preempted.
+func (a *Allocation) MarkPreempted() {
+	a.Lock()
+	defer a.Unlock()
+	a.preempted = true
+}
+
+// IsPreempted returns whether the allocation has been marked for preemption or not.
+func (a *Allocation) IsPreempted() bool {
+	a.Lock()
+	defer a.Unlock()
+	return a.preempted
+}
+
 // CloneAllocationTags clones a tag map for safe copying
 func CloneAllocationTags(tags map[string]string) map[string]string {
 	result := make(map[string]string)
