@@ -37,6 +37,10 @@ func TestNewNodeCollection(t *testing.T) {
 		t.Fatal("NewNodeCollection should return baseNodeCollection")
 	}
 	assert.Equal(t, "test", bc.Partition, "node collection should have matching partition")
+
+	if nc.GetNodeSortingPolicy().PolicyType().String() != policies.FairSortPolicy.String() {
+		t.Errorf("Node default Policy: want %s, got %s", nc.GetNodeSortingPolicy().PolicyType().String(), policies.FairSortPolicy.String())
+	}
 }
 
 func initBaseCollection() *baseNodeCollection {
