@@ -20,14 +20,13 @@ package cache
 
 import (
 	"fmt"
+	"reflect"
 	"sort"
 	"strings"
 	"sync"
 	"testing"
 	"time"
 
-	"github.com/apache/yunikorn-k8shim/pkg/dispatcher"
-
 	"gotest.tools/assert"
 	is "gotest.tools/assert/cmp"
 	v1 "k8s.io/api/core/v1"
@@ -44,6 +43,7 @@ import (
 	"github.com/apache/yunikorn-k8shim/pkg/common/events"
 	"github.com/apache/yunikorn-k8shim/pkg/common/utils"
 	"github.com/apache/yunikorn-k8shim/pkg/conf"
+	"github.com/apache/yunikorn-k8shim/pkg/dispatcher"
 	"github.com/apache/yunikorn-scheduler-interface/lib/go/api"
 	siCommon "github.com/apache/yunikorn-scheduler-interface/lib/go/common"
 	"github.com/apache/yunikorn-scheduler-interface/lib/go/si"
@@ -55,17 +55,18 @@ type recorderTime struct {
 }
 
 func TestNewApplication(t *testing.T) {
-	app := NewApplication("app00001", "root.queue", "testuser", map[string]string{}, newMockSchedulerAPI())
+	app := NewApplication("app00001", "root.queue", "testuser", testGroups, map[string]string{}, newMockSchedulerAPI())
 	assert.Equal(t, app.GetApplicationID(), "app00001")
 	assert.Equal(t, app.GetApplicationState(), ApplicationStates().New)
 	assert.Equal(t, app.partition, constants.DefaultPartition)
 	assert.Equal(t, len(app.taskMap), 0)
 	assert.Equal(t, app.GetApplicationState(), ApplicationStates().New)
 	assert.Equal(t, app.queue, "root.queue")
+	assert.Assert(t, reflect.DeepEqual(app.groups, []string{"dev", "yunikorn"}))
 }
 
 func TestSubmitApplication(t *testing.T) {
-	app := NewApplication("app00001", "root.abc", "testuser", map[string]string{}, newMockSchedulerAPI())
+	app := NewApplication("app00001", "root.abc", "testuser", testGroups, map[string]string{}, newMockSchedulerAPI())
 	err := app.handle(NewSubmitApplicationEvent(app.applicationID))
 	assert.NilError(t, err)
 	assertAppState(t, app, ApplicationStates().Submitted, 10*time.Second)
@@ -88,7 +89,7 @@ func TestRunApplication(t *testing.T) {
 		return nil
 	}
 
-	app := NewApplication("app00001", "root.abc", "testuser", map[string]string{}, ms)
+	app := NewApplication("app00001", "root.abc", "testuser", testGroups, map[string]string{}, ms)
 
 	// app must be submitted before being able to run
 	err := app.handle(NewRunApplicationEvent(app.applicationID))
@@ -166,7 +167,7 @@ func TestFailApplication(t *testing.T) {
 		},
 	}
 	appID := "app-test-001"
-	app := NewApplication(appID, "root.abc", "testuser", map[string]string{}, ms)
+	app := NewApplication(appID, "root.abc", "testuser", testGroups, map[string]string{}, ms)
 	task1 := NewTask("task01", app, context, pod)
 	task2 := NewTask("task02", app, context, pod)
 	task3 := NewTask("task03", app, context, pod)
@@ -192,7 +193,7 @@ func TestFailApplication(t *testing.T) {
 	// reset time to 0
 	rt.time = 0
 	appID2 := "app-test-002"
-	app2 := NewApplication(appID2, "root.abc", "testuser", map[string]string{}, ms)
+	app2 := NewApplication(appID2, "root.abc", "testuser", testGroups, map[string]string{}, ms)
 	app2.SetState(ApplicationStates().New)
 	err = app2.handle(NewFailApplicationEvent(app2.applicationID, errMess))
 	if err == nil {
@@ -288,7 +289,7 @@ func TestSetUnallocatedPodsToFailedWhenFailApplication(t *testing.T) {
 	})
 	assert.NilError(t, err)
 	appID := "app-test-001"
-	app := NewApplication(appID, "root.abc", "testuser", map[string]string{}, ms)
+	app := NewApplication(appID, "root.abc", "testuser", testGroups, map[string]string{}, ms)
 	task1 := NewTask("task01", app, context, pod1)
 	task2 := NewTaskPlaceholder("task02", app, context, pod2)
 	task3 := NewTask("task03", app, context, pod3)
@@ -383,7 +384,7 @@ func TestSetUnallocatedPodsToFailedWhenRejectApplication(t *testing.T) {
 	})
 	assert.NilError(t, err)
 	appID := "app-test-001"
-	app := NewApplication(appID, "root.abc", "testuser", map[string]string{}, ms)
+	app := NewApplication(appID, "root.abc", "testuser", testGroups, map[string]string{}, ms)
 	task1 := NewTask("task01", app, context, pod1)
 	task2 := NewTask("task02", app, context, pod2)
 	task1.sm.SetState(TaskStates().Pending)
@@ -447,7 +448,7 @@ func TestReleaseAppAllocation(t *testing.T) {
 	}
 	appID := "app-test-001"
 	UUID := "testUUID001"
-	app := NewApplication(appID, "root.abc", "testuser", map[string]string{}, ms)
+	app := NewApplication(appID, "root.abc", "testuser", testGroups, map[string]string{}, ms)
 	task := NewTask("task01", app, context, pod)
 	app.addTask(task)
 	task.allocationUUID = UUID
@@ -480,7 +481,7 @@ func newMockSchedulerAPI() *mockSchedulerAPI {
 		UpdateNodeFn: func(request *si.NodeRequest) error {
 			return nil
 		},
-		UpdateConfigurationFn: func(clusterID string) error {
+		UpdateConfigurationFn: func(request *si.UpdateConfigurationRequest) error {
 			return nil
 		},
 	}
@@ -493,7 +494,7 @@ type mockSchedulerAPI struct {
 	UpdateAllocationFn    func(request *si.AllocationRequest) error
 	UpdateApplicationFn   func(request *si.ApplicationRequest) error
 	UpdateNodeFn          func(request *si.NodeRequest) error
-	UpdateConfigurationFn func(clusterID string) error
+	UpdateConfigurationFn func(request *si.UpdateConfigurationRequest) error
 }
 
 func (ms *mockSchedulerAPI) RegisterResourceManager(request *si.RegisterResourceManagerRequest,
@@ -513,7 +514,7 @@ func (ms *mockSchedulerAPI) UpdateNode(request *si.NodeRequest) error {
 	return ms.UpdateNodeFn(request)
 }
 
-func (ms *mockSchedulerAPI) UpdateConfiguration(clusterID string) error {
+func (ms *mockSchedulerAPI) UpdateConfiguration(request *si.UpdateConfigurationRequest) error {
 	return nil
 }
 
@@ -533,8 +534,8 @@ func assertAppState(t *testing.T, app *Application, expectedState string, durati
 func TestGetNonTerminatedTaskAlias(t *testing.T) {
 	context := initContextForTest()
 	appID := "app00001"
-	app := NewApplication(appID, "root.a", "testuser", map[string]string{}, newMockSchedulerAPI())
-	context.applications[appID] = app
+	app := NewApplication(appID, "root.a", "testuser", testGroups, map[string]string{}, newMockSchedulerAPI())
+	context.addApplication(app)
 	// app doesn't have any task
 	res := app.getNonTerminatedTaskAlias()
 	assert.Equal(t, len(res), 0)
@@ -593,7 +594,7 @@ func TestGetNonTerminatedTaskAlias(t *testing.T) {
 }
 
 func TestSetTaskGroupsAndSchedulingPolicy(t *testing.T) {
-	app := NewApplication("app01", "root.a", "test-user", map[string]string{}, newMockSchedulerAPI())
+	app := NewApplication("app01", "root.a", "test-user", testGroups, map[string]string{}, newMockSchedulerAPI())
 	assert.Assert(t, app.getSchedulingPolicy().Type == "")
 	assert.Equal(t, len(app.getTaskGroups()), 0)
 
@@ -717,8 +718,8 @@ func TestTryReserve(t *testing.T) {
 
 	// create a new app
 	app := NewApplication("app00001", "root.abc", "test-user",
-		map[string]string{}, mockedAPIProvider.GetAPIs().SchedulerAPI)
-	context.applications[app.applicationID] = app
+		testGroups, map[string]string{}, mockedAPIProvider.GetAPIs().SchedulerAPI)
+	context.addApplication(app)
 
 	// set app scheduling policy
 	app.setSchedulingPolicy(v1alpha1.SchedulingPolicy{
@@ -792,8 +793,8 @@ func TestTryReservePostRestart(t *testing.T) {
 
 	// create a new app
 	app := NewApplication("app00001", "root.abc", "test-user",
-		map[string]string{}, mockedAPIProvider.GetAPIs().SchedulerAPI)
-	context.applications[app.applicationID] = app
+		testGroups, map[string]string{}, mockedAPIProvider.GetAPIs().SchedulerAPI)
+	context.addApplication(app)
 
 	// set taskGroups
 	app.setTaskGroups([]v1alpha1.TaskGroup{
@@ -904,7 +905,7 @@ func TestTriggerAppRecovery(t *testing.T) {
 	}
 
 	app := NewApplication("app00001", "root.abc", "test-user",
-		map[string]string{}, mockScheduler)
+		testGroups, map[string]string{}, mockScheduler)
 	app.placeholderAsk = &si.Resource{
 		Resources: map[string]*si.Quantity{
 			"memory": {Value: 100},
@@ -936,7 +937,7 @@ func TestTriggerAppRecovery(t *testing.T) {
 
 	// Trigger app recovery should be failed if the app already leaves New state
 	app = NewApplication("app00001", "root.abc", "test-user",
-		map[string]string{}, newMockSchedulerAPI())
+		testGroups, map[string]string{}, newMockSchedulerAPI())
 	err = app.handle(NewSubmitApplicationEvent(app.applicationID))
 	assert.NilError(t, err)
 	assertAppState(t, app, ApplicationStates().Submitted, 3*time.Second)
@@ -946,14 +947,14 @@ func TestTriggerAppRecovery(t *testing.T) {
 
 func TestSkipReservationStage(t *testing.T) {
 	context := initContextForTest()
-	app := NewApplication("app00001", "root.queue", "test-user", map[string]string{}, newMockSchedulerAPI())
+	app := NewApplication("app00001", "root.queue", "test-user", testGroups, map[string]string{}, newMockSchedulerAPI())
 	app.addTask(NewTask("task0001", app, context, &v1.Pod{}))
 	skip := app.skipReservationStage()
 	assert.Equal(t, skip, true, "expected to skip reservation because there is no task groups defined")
 
 	// app has task groups defined, and contains 2 tasks, 1 Pending and 1 Allocated
 	// expect: skip reservation
-	app = NewApplication("app00001", "root.queue", "test-user", map[string]string{}, newMockSchedulerAPI())
+	app = NewApplication("app00001", "root.queue", "test-user", testGroups, map[string]string{}, newMockSchedulerAPI())
 	task1 := NewTask("task0001", app, context, &v1.Pod{})
 	task1.sm.SetState(TaskStates().New)
 	task2 := NewTask("task0002", app, context, &v1.Pod{})
@@ -975,7 +976,7 @@ func TestSkipReservationStage(t *testing.T) {
 
 	// app has task groups defined, and contains 2 tasks, both are New
 	// expect: do not skip reservation
-	app = NewApplication("app00001", "root.queue", "test-user", map[string]string{}, newMockSchedulerAPI())
+	app = NewApplication("app00001", "root.queue", "test-user", testGroups, map[string]string{}, newMockSchedulerAPI())
 	task1 = NewTask("task0001", app, context, &v1.Pod{})
 	task1.sm.SetState(TaskStates().New)
 	task2 = NewTask("task0002", app, context, &v1.Pod{})
@@ -1022,7 +1023,7 @@ func TestReleaseAppAllocationInFailingState(t *testing.T) {
 	}
 	appID := "app-test-001"
 	UUID := "testUUID001"
-	app := NewApplication(appID, "root.abc", "testuser", map[string]string{}, ms)
+	app := NewApplication(appID, "root.abc", "testuser", testGroups, map[string]string{}, ms)
 	task := NewTask("task01", app, context, pod)
 	app.addTask(task)
 	task.allocationUUID = UUID
@@ -1071,7 +1072,7 @@ func TestResumingStateTransitions(t *testing.T) {
 
 	// create a new app
 	app := NewApplication("app00001", "root.abc", "test-user",
-		map[string]string{}, mockedAPIProvider.GetAPIs().SchedulerAPI)
+		testGroups, map[string]string{}, mockedAPIProvider.GetAPIs().SchedulerAPI)
 	task1 := NewTask("task0001", app, context, &v1.Pod{})
 	task1.sm.SetState(TaskStates().New)
 	task2 := NewTask("task0002", app, context, &v1.Pod{})
@@ -1082,7 +1083,7 @@ func TestResumingStateTransitions(t *testing.T) {
 	app.addTask(task2)
 	UUID := "testUUID001"
 	task1.allocationUUID = UUID
-	context.applications[app.applicationID] = app
+	context.addApplication(app)
 
 	// Set app state to "reserving"
 	app.SetState(ApplicationStates().Reserving)
@@ -1114,7 +1115,7 @@ func TestResumingStateTransitions(t *testing.T) {
 
 func TestGetPlaceholderTasks(t *testing.T) {
 	context := initContextForTest()
-	app := NewApplication(appID, "root.a", "testuser", map[string]string{}, newMockSchedulerAPI())
+	app := NewApplication(appID, "root.a", "testuser", testGroups, map[string]string{}, newMockSchedulerAPI())
 	task1 := NewTask("task0001", app, context, &v1.Pod{})
 	task1.placeholder = true
 	task2 := NewTask("task0002", app, context, &v1.Pod{})
@@ -1204,7 +1205,7 @@ func TestPlaceholderTimeoutEvents(t *testing.T) {
 	appID := "app00001"
 	UUID := "UID-POD-00002"
 
-	context.applications[appID] = app
+	context.addApplication(app)
 	task1 := context.AddTask(&interfaces.AddTaskRequest{
 		Metadata: interfaces.TaskMetadata{
 			ApplicationID: "app00001",
@@ -1255,3 +1256,84 @@ func TestPlaceholderTimeoutEvents(t *testing.T) {
 	}, 5*time.Millisecond, 20*time.Millisecond)
 	assert.NilError(t, err, "event should have been emitted")
 }
+
+func TestApplication_onReservationStateChange(t *testing.T) {
+	context := initContextForTest()
+	dispatcher.RegisterEventHandler(dispatcher.EventTypeApp, context.ApplicationEventHandler())
+	dispatcher.Start()
+	defer dispatcher.Stop()
+
+	app := NewApplication(appID, "root.a", "testuser", testGroups, map[string]string{}, newMockSchedulerAPI())
+	context.addApplication(app)
+
+	app.sm.SetState("Accepted")
+	app.onReservationStateChange()
+	// app goes to running (no taskgroups defined)
+	assertAppState(t, app, ApplicationStates().Running, 1*time.Second)
+
+	// set taskGroups
+	app.setTaskGroups([]v1alpha1.TaskGroup{
+		{
+			Name:      "test-group-1",
+			MinMember: 1,
+			MinResource: map[string]resource.Quantity{
+				v1.ResourceCPU.String():    resource.MustParse("500m"),
+				v1.ResourceMemory.String(): resource.MustParse("500Mi"),
+			},
+		},
+		{
+			Name:      "test-group-2",
+			MinMember: 2,
+			MinResource: map[string]resource.Quantity{
+				v1.ResourceCPU.String():    resource.MustParse("500m"),
+				v1.ResourceMemory.String(): resource.MustParse("500Mi"),
+			},
+		},
+	})
+	app.sm.SetState("Accepted")
+	app.onReservationStateChange()
+	// app stays in accepted with taskgroup defined
+	assertAppState(t, app, ApplicationStates().Accepted, 1*time.Second)
+
+	task1 := NewTask("task0001", app, context, &v1.Pod{})
+	task1.setTaskGroupName("test-group-1")
+	task1.placeholder = true
+	task2 := NewTask("task0002", app, context, &v1.Pod{})
+	task2.setTaskGroupName("test-group-2")
+	task2.placeholder = true
+	// not placeholder, unknown group
+	task3 := NewTask("task0003", app, context, &v1.Pod{})
+	task3.setTaskGroupName("unknown")
+
+	app.addTask(task1)
+	app.addTask(task2)
+	app.addTask(task3)
+
+	// app stays in accepted with taskgroups defined none bound
+	app.onReservationStateChange()
+	assertAppState(t, app, ApplicationStates().Accepted, 1*time.Second)
+
+	// app stays in accepted with taskgroups defined one unknown taskgroup not a placeholder
+	// all tasks bound
+	task1.sm.SetState("Bound")
+	task2.sm.SetState("Bound")
+	task3.sm.SetState("Bound")
+	app.onReservationStateChange()
+	assertAppState(t, app, ApplicationStates().Accepted, 1*time.Second)
+
+	// app stays in accepted with taskgroups defined one unknown taskgroup
+	task3.placeholder = true
+	app.onReservationStateChange()
+	assertAppState(t, app, ApplicationStates().Accepted, 1*time.Second)
+
+	// app moves to running
+	task3.setTaskGroupName("test-group-2")
+	app.onReservationStateChange()
+	assertAppState(t, app, ApplicationStates().Running, 1*time.Second)
+}
+
+func (ctx *Context) addApplication(app *Application) {
+	ctx.lock.Lock()
+	defer ctx.lock.Unlock()
+	ctx.applications[app.applicationID] = app
+}
