@@ -19,7 +19,6 @@
 package cache
 
 import (
-	ctx "context"
 	"encoding/json"
 	"fmt"
 	"strings"
@@ -28,6 +27,7 @@ import (
 
 	"gotest.tools/assert"
 	v1 "k8s.io/api/core/v1"
+	schedulingv1 "k8s.io/api/scheduling/v1"
 	apis "k8s.io/apimachinery/pkg/apis/meta/v1"
 	"k8s.io/apimachinery/pkg/types"
 	"k8s.io/client-go/tools/cache"
@@ -47,10 +47,20 @@ import (
 	"github.com/apache/yunikorn-scheduler-interface/lib/go/si"
 )
 
+var (
+	testGroups = []string{"dev", "yunikorn"}
+)
+
 func initContextForTest() *Context {
+	ctx, _ := initContextAndAPIProviderForTest()
+	return ctx
+}
+
+func initContextAndAPIProviderForTest() (*Context, *client.MockedAPIProvider) {
 	conf.GetSchedulerConf().SetTestMode(true)
-	context := NewContext(client.NewMockedAPIProvider(false))
-	return context
+	apis := client.NewMockedAPIProvider(false)
+	context := NewContext(apis)
+	return context, apis
 }
 
 func newPodHelper(name, namespace, podUID, nodeName string, podPhase v1.PodPhase) *v1.Pod {
@@ -147,9 +157,9 @@ func TestRemoveApplication(t *testing.T) {
 	appID1 := "app00001"
 	appID2 := "app00002"
 	appID3 := "app00003"
-	app1 := NewApplication(appID1, "root.a", "testuser", map[string]string{}, newMockSchedulerAPI())
-	app2 := NewApplication(appID2, "root.b", "testuser", map[string]string{}, newMockSchedulerAPI())
-	app3 := NewApplication(appID3, "root.c", "testuser", map[string]string{}, newMockSchedulerAPI())
+	app1 := NewApplication(appID1, "root.a", "testuser", testGroups, map[string]string{}, newMockSchedulerAPI())
+	app2 := NewApplication(appID2, "root.b", "testuser", testGroups, map[string]string{}, newMockSchedulerAPI())
+	app3 := NewApplication(appID3, "root.c", "testuser", testGroups, map[string]string{}, newMockSchedulerAPI())
 	context.applications[appID1] = app1
 	context.applications[appID2] = app2
 	context.applications[appID3] = app3
@@ -217,8 +227,8 @@ func TestRemoveApplicationInternal(t *testing.T) {
 	context := initContextForTest()
 	appID1 := "app00001"
 	appID2 := "app00002"
-	app1 := NewApplication(appID1, "root.a", "testuser", map[string]string{}, newMockSchedulerAPI())
-	app2 := NewApplication(appID2, "root.b", "testuser", map[string]string{}, newMockSchedulerAPI())
+	app1 := NewApplication(appID1, "root.a", "testuser", testGroups, map[string]string{}, newMockSchedulerAPI())
+	app2 := NewApplication(appID2, "root.b", "testuser", testGroups, map[string]string{}, newMockSchedulerAPI())
 	context.applications[appID1] = app1
 	context.applications[appID2] = app2
 	assert.Equal(t, len(context.applications), 2)
@@ -761,9 +771,9 @@ func TestGetTask(t *testing.T) {
 	appID1 := "app00001"
 	appID2 := "app00002"
 	appID3 := "app00003"
-	app1 := NewApplication(appID1, "root.a", "testuser", map[string]string{}, newMockSchedulerAPI())
-	app2 := NewApplication(appID2, "root.b", "testuser", map[string]string{}, newMockSchedulerAPI())
-	app3 := NewApplication(appID3, "root.c", "testuser", map[string]string{}, newMockSchedulerAPI())
+	app1 := NewApplication(appID1, "root.a", "testuser", testGroups, map[string]string{}, newMockSchedulerAPI())
+	app2 := NewApplication(appID2, "root.b", "testuser", testGroups, map[string]string{}, newMockSchedulerAPI())
+	app3 := NewApplication(appID3, "root.c", "testuser", testGroups, map[string]string{}, newMockSchedulerAPI())
 	context.applications[appID1] = app1
 	context.applications[appID2] = app2
 	context.applications[appID3] = app3
@@ -1080,71 +1090,6 @@ func TestAddApplicationsWithTags(t *testing.T) {
 	assert.Equal(t, parentQueue, "root.test")
 }
 
-func TestFindYKConfigMap(t *testing.T) {
-	goodYKConfigmap := v1.ConfigMap{
-		ObjectMeta: apis.ObjectMeta{
-			Name:   constants.DefaultConfigMapName,
-			Labels: map[string]string{"app": "yunikorn", "label2": "value2"},
-		},
-		Data: map[string]string{"queues.yaml": "OldData"},
-	}
-	randomConfigMap := v1.ConfigMap{
-		ObjectMeta: apis.ObjectMeta{
-			Name: "configMap",
-		},
-	}
-	testCases := []struct {
-		name          string
-		expectedError bool
-		configMaps    []*v1.ConfigMap
-	}{
-		{"Nil configmaps", true, nil},
-		{"Empty configmaps", true, []*v1.ConfigMap{}},
-		{"Yunikorn configmap found", false, []*v1.ConfigMap{&goodYKConfigmap, &randomConfigMap}},
-	}
-	for _, tc := range testCases {
-		t.Run(tc.name, func(t *testing.T) {
-			configMap, err := findYKConfigMap(tc.configMaps)
-			if tc.expectedError {
-				assert.Assert(t, err != nil, "Error is expected")
-			} else {
-				assert.Assert(t, configMap.Name == constants.DefaultConfigMapName, "Returned configmap is wrong")
-				assert.Assert(t, len(configMap.Data) == 1, "Returned configmap has unexpected data")
-				assert.Assert(t, configMap.Name == constants.DefaultConfigMapName, "Returned configmap is wrong")
-				assert.Assert(t, configMap.Data["queues.yaml"] == "OldData", "Old configmap value is wrong")
-			}
-		})
-	}
-}
-
-func TestSaveConfigmap(t *testing.T) {
-	// YK configmap not found
-	context := initContextForTest()
-	newConf := si.UpdateConfigurationRequest{
-		Configs: "newConfig",
-	}
-	resp := context.SaveConfigmap(&newConf)
-	assert.Equal(t, false, resp.Success, "Successful update expected")
-	assert.Assert(t, strings.Contains(resp.Reason, "not found"), "Unexpected reason returned")
-
-	// successful update
-	configMaps, err := context.apiProvider.GetAPIs().ConfigMapInformer.Lister().List(nil)
-	assert.NilError(t, err, "No error expected")
-	for _, c := range configMaps {
-		_, err := context.apiProvider.GetAPIs().KubeClient.GetClientSet().CoreV1().ConfigMaps(c.Namespace).
-			Create(ctx.Background(), c, apis.CreateOptions{})
-		assert.NilError(t, err, "No error expected")
-	}
-	resp = context.SaveConfigmap(&newConf)
-	assert.Equal(t, true, resp.Success, "Successful update expected")
-
-	// hot-refresh enabled
-	context.apiProvider.GetAPIs().Conf.EnableConfigHotRefresh = true
-	resp = context.SaveConfigmap(&newConf)
-	assert.Equal(t, false, resp.Success, "Failure is expected")
-	assert.Assert(t, strings.Contains(resp.Reason, "hot-refresh is enabled"), "Unexpected reason")
-}
-
 func TestPendingPodAllocations(t *testing.T) {
 	context := initContextForTest()
 	context.SetPluginMode(true)
@@ -1244,3 +1189,131 @@ func TestPendingPodAllocations(t *testing.T) {
 		t.Fatalf("pending pod allocation still exists after removal")
 	}
 }
+
+func TestGetStateDump(t *testing.T) {
+	context := initContextForTest()
+
+	pod1 := &v1.Pod{
+		TypeMeta: apis.TypeMeta{
+			Kind:       "Pod",
+			APIVersion: "v1",
+		},
+		ObjectMeta: apis.ObjectMeta{
+			Namespace: "default",
+			Name:      "yunikorn-test-00001",
+			UID:       "UID-00001",
+		},
+		Spec: v1.PodSpec{SchedulerName: "yunikorn"},
+	}
+	context.addPodToCache(pod1)
+
+	stateDumpStr, err := context.GetStateDump()
+	assert.NilError(t, err, "error during state dump")
+	var stateDump = make(map[string]interface{})
+	err = json.Unmarshal([]byte(stateDumpStr), &stateDump)
+	assert.NilError(t, err, "unable to parse state dump")
+
+	cacheObj, ok := stateDump["cache"]
+	assert.Assert(t, ok, "cache not found")
+	cache, ok := cacheObj.(map[string]interface{})
+	assert.Assert(t, ok, "unable to cast cache")
+
+	podsObj, ok := cache["pods"]
+	assert.Assert(t, ok, "pods not found")
+	pods, ok := podsObj.(map[string]interface{})
+	assert.Assert(t, ok, "unable to cast pods")
+
+	podObj, ok := pods["default/yunikorn-test-00001"]
+	assert.Assert(t, ok, "pod not found")
+	pod, ok := podObj.(map[string]interface{})
+	assert.Assert(t, ok, "unable to cast pod")
+
+	uidObj, ok := pod["uid"]
+	assert.Assert(t, ok, "uid not found")
+	uid, ok := uidObj.(string)
+	assert.Assert(t, ok, "Unable to cast uid")
+
+	assert.Equal(t, string(pod1.UID), uid, "wrong uid")
+}
+
+func TestFilterPriorityClasses(t *testing.T) {
+	context := initContextForTest()
+	policy := v1.PreemptLowerPriority
+	pc := &schedulingv1.PriorityClass{
+		ObjectMeta: apis.ObjectMeta{
+			Name: "pc-test",
+		},
+		Value:            100,
+		GlobalDefault:    false,
+		PreemptionPolicy: &policy,
+	}
+
+	assert.Check(t, !context.filterPriorityClasses(nil), "nil object was allowed")
+	assert.Check(t, !context.filterPriorityClasses("test"), "non-pc class was allowed")
+	assert.Check(t, context.filterPriorityClasses(pc), "pc was ignored")
+}
+
+func TestAddPriorityClass(t *testing.T) {
+	context := initContextForTest()
+	policy := v1.PreemptLowerPriority
+	pc := &schedulingv1.PriorityClass{
+		ObjectMeta: apis.ObjectMeta{
+			Name: "pc-test",
+		},
+		Value:            100,
+		GlobalDefault:    false,
+		PreemptionPolicy: &policy,
+	}
+	context.addPriorityClass(pc)
+	result := context.schedulerCache.GetPriorityClass("pc-test")
+	assert.Assert(t, result != nil)
+	assert.Equal(t, result.Value, int32(100))
+}
+
+func TestUpdatePriorityClass(t *testing.T) {
+	context := initContextForTest()
+	policy := v1.PreemptLowerPriority
+	pc := &schedulingv1.PriorityClass{
+		ObjectMeta: apis.ObjectMeta{
+			Name: "pc-test",
+		},
+		Value:            100,
+		GlobalDefault:    false,
+		PreemptionPolicy: &policy,
+	}
+	policy2 := v1.PreemptNever
+	pc2 := &schedulingv1.PriorityClass{
+		ObjectMeta: apis.ObjectMeta{
+			Name: "pc-test",
+		},
+		Value:            200,
+		GlobalDefault:    false,
+		PreemptionPolicy: &policy2,
+	}
+
+	context.addPriorityClass(pc)
+	context.updatePriorityClass(pc, pc2)
+	result := context.schedulerCache.GetPriorityClass("pc-test")
+	assert.Assert(t, result != nil)
+	assert.Equal(t, result.Value, int32(200))
+}
+
+func TestDeletePriorityClass(t *testing.T) {
+	context := initContextForTest()
+	policy := v1.PreemptLowerPriority
+	pc := &schedulingv1.PriorityClass{
+		ObjectMeta: apis.ObjectMeta{
+			Name: "pc-test",
+		},
+		Value:            100,
+		GlobalDefault:    false,
+		PreemptionPolicy: &policy,
+	}
+
+	context.addPriorityClass(pc)
+	result := context.schedulerCache.GetPriorityClass("pc-test")
+	assert.Assert(t, result != nil)
+	context.deletePriorityClass(pc)
+	result = context.schedulerCache.GetPriorityClass("pc-test")
+	assert.Assert(t, result == nil)
+}
