@@ -20,6 +20,7 @@ package cache
 
 import (
 	"fmt"
+	"strconv"
 	"sync"
 	"time"
 
@@ -27,6 +28,7 @@ import (
 
 	"github.com/apache/yunikorn-k8shim/pkg/appmgmt/interfaces"
 	"github.com/apache/yunikorn-k8shim/pkg/common"
+	"github.com/apache/yunikorn-k8shim/pkg/common/constants"
 	"github.com/apache/yunikorn-k8shim/pkg/common/events"
 	"github.com/apache/yunikorn-k8shim/pkg/common/utils"
 	"github.com/apache/yunikorn-k8shim/pkg/dispatcher"
@@ -187,6 +189,10 @@ func (task *Task) UpdateTaskPodStatus(pod *v1.Pod) (*v1.Pod, error) {
 	return task.context.apiProvider.GetAPIs().KubeClient.UpdateStatus(pod)
 }
 
+func (task *Task) UpdateTaskPod(pod *v1.Pod, podMutator func(pod *v1.Pod)) (*v1.Pod, error) {
+	return task.context.apiProvider.GetAPIs().KubeClient.UpdatePod(pod, podMutator)
+}
+
 func (task *Task) isTerminated() bool {
 	for _, states := range TaskStates().Terminated {
 		if task.GetTaskState() == states {
@@ -262,9 +268,43 @@ func (task *Task) handleFailEvent(reason string, err bool) {
 		zap.String("reason", reason))
 }
 
+func (task *Task) isPreemptSelfAllowed() bool {
+	value := utils.GetPodAnnotationValue(task.pod, constants.AnnotationAllowPreemption)
+	switch value {
+	case constants.True:
+		return true
+	case constants.False:
+		return false
+	default:
+		return task.context.IsPreemptSelfAllowed(task.pod.Spec.PriorityClassName)
+	}
+}
+
+func (task *Task) isPreemptOtherAllowed() bool {
+	policy := task.pod.Spec.PreemptionPolicy
+	if policy == nil {
+		return true
+	}
+	switch *policy {
+	case v1.PreemptNever:
+		return false
+	case v1.PreemptLowerPriority:
+		return true
+	default:
+		return true
+	}
+}
+
 func (task *Task) handleSubmitTaskEvent() {
 	log.Logger().Debug("scheduling pod",
 		zap.String("podName", task.pod.Name))
+
+	// build preemption policy
+	preemptionPolicy := &si.PreemptionPolicy{
+		AllowPreemptSelf:  task.isPreemptSelfAllowed(),
+		AllowPreemptOther: task.isPreemptOtherAllowed(),
+	}
+
 	// convert the request
 	rr := common.CreateAllocationRequestForTask(
 		task.applicationID,
@@ -273,7 +313,8 @@ func (task *Task) handleSubmitTaskEvent() {
 		task.placeholder,
 		task.taskGroupName,
 		task.pod,
-		task.originator)
+		task.originator,
+		preemptionPolicy)
 	log.Logger().Debug("send update request", zap.String("request", rr.String()))
 	if err := task.context.apiProvider.GetAPIs().SchedulerAPI.UpdateAllocation(&rr); err != nil {
 		log.Logger().Debug("failed to send scheduling request to scheduler", zap.Error(err))
@@ -399,12 +440,19 @@ func (task *Task) postTaskBound() {
 		// the pod status explicitly, when there is a status change, the default scheduler will
 		// move the pod back to the active queue immediately.
 		podCopy := task.pod.DeepCopy()
-		podCopy.Status = v1.PodStatus{
-			Phase:   podCopy.Status.Phase,
-			Reason:  "QuotaApproved",
-			Message: "pod fits into the queue quota and it is ready for scheduling",
-		}
-		if _, err := task.UpdateTaskPodStatus(podCopy); err != nil {
+		if _, err := task.UpdateTaskPod(podCopy, func(pod *v1.Pod) {
+			pod.Status = v1.PodStatus{
+				Phase:   podCopy.Status.Phase,
+				Reason:  "Pending",
+				Message: "pod fits into the queue quota and it is ready for scheduling",
+			}
+
+			// this is a bit of a hack, but ensures that the default scheduler detects the pod as having changed
+			if pod.Annotations == nil {
+				pod.Annotations = make(map[string]string)
+			}
+			pod.Annotations["yunikorn.apache.org/scheduled-at"] = strconv.FormatInt(time.Now().UnixNano(), 10)
+		}); err != nil {
 			log.Logger().Warn("failed to update pod status", zap.Error(err))
 		}
 	}
