@@ -25,7 +25,6 @@ import (
 
 	"github.com/apache/yunikorn-k8shim/pkg/conf"
 	"github.com/apache/yunikorn-k8shim/pkg/schedulerplugin"
-	pluginconf "github.com/apache/yunikorn-k8shim/pkg/schedulerplugin/conf"
 )
 
 var (
@@ -34,9 +33,6 @@ var (
 )
 
 func main() {
-	// override the default config handling when in plugin mode
-	conf.SetSchedulerConfFactory(pluginconf.NewSchedulerConf)
-
 	conf.BuildVersion = version
 	conf.BuildDate = date
 	conf.IsPluginVersion = true
@@ -44,8 +40,6 @@ func main() {
 	command := app.NewSchedulerCommand(
 		app.WithPlugin(schedulerplugin.SchedulerPluginName, schedulerplugin.NewSchedulerPlugin))
 
-	pluginconf.InitCliFlagSet(command)
-
 	if err := command.Execute(); err != nil {
 		os.Exit(1)
 	}
