@@ -18,6 +18,10 @@
 
 package constants
 
+// Common
+const True = "true"
+const False = "false"
+
 // Cluster
 const DefaultNodeAttributeHostNameKey = "si.io/hostname"
 const DefaultNodeAttributeRackNameKey = "si.io/rackname"
@@ -30,11 +34,13 @@ const LabelApplicationID = "applicationId"
 const AnnotationApplicationID = "yunikorn.apache.org/app-id"
 const LabelQueueName = "queue"
 const AnnotationQueueName = "yunikorn.apache.org/queue"
+const AnnotationParentQueue = "yunikorn.apache.org/parentqueue"
 const LabelDisableStateAware = "disableStateAware"
 const ApplicationDefaultQueue = "root.sandbox"
 const DefaultPartition = "default"
 const AppTagNamespace = "namespace"
 const AppTagNamespaceParentQueue = "namespace.parentqueue"
+const AppTagImagePullSecrets = "imagePullSecrets"
 const DefaultAppNamespace = "default"
 const DefaultUserLabel = "yunikorn.apache.org/username"
 const DefaultUser = "nobody"
@@ -45,7 +51,8 @@ const SparkLabelRole = "spark-role"
 const SparkLabelRoleDriver = "driver"
 
 // Configuration
-const DefaultConfigMapName = "yunikorn-configs"
+const ConfigMapName = "yunikorn-configs"
+const DefaultConfigMapName = "yunikorn-defaults"
 const SchedulerName = "yunikorn"
 
 // OwnerReferences
@@ -55,7 +62,7 @@ const DaemonSetType = "DaemonSet"
 const AppManagerHandlerName = "yunikorn-app"
 
 // Gang scheduling
-const PlaceholderContainerImage = "k8s.gcr.io/pause"
+const PlaceholderContainerImage = "registry.k8s.io/pause:3.7"
 const PlaceholderContainerName = "pause"
 const PlaceholderPodRestartPolicy = "Never"
 const LabelPlaceholderFlag = "placeholder"
@@ -72,3 +79,18 @@ var SchedulingPolicyStyleParamValues = map[string]string{"Hard": "Hard", "Soft":
 
 const ApplicationInsufficientResourcesFailure = "ResourceReservationTimeout"
 const ApplicationRejectedFailure = "ApplicationRejected"
+
+// namespace.max.* (Retaining for backwards compatibility. Need to be removed in next major release)
+const CPUQuota = "yunikorn.apache.org/namespace.max.cpu"
+const MemQuota = "yunikorn.apache.org/namespace.max.memory"
+
+// NamespaceQuota Namespace Quota
+const NamespaceQuota = "yunikorn.apache.org/namespace.quota"
+
+// Preemption
+const AnnotationAllowPreemption = "yunikorn.apache.org/allow-preemption"
+
+// Admission Controller pod label update constants
+const AutoGenAppPrefix = "yunikorn"
+const AutoGenAppSuffix = "autogen"
+const DefaultQueue = "root.default"
