@@ -31,22 +31,27 @@ import (
 	"time"
 
 	"github.com/onsi/ginkgo"
-	"k8s.io/apimachinery/pkg/util/wait"
-	podutil "k8s.io/kubernetes/pkg/api/v1/pod"
-
+	appsv1 "k8s.io/api/apps/v1"
 	batchv1 "k8s.io/api/batch/v1"
 	v1 "k8s.io/api/core/v1"
 	authv1 "k8s.io/api/rbac/v1"
+	schedulingv1 "k8s.io/api/scheduling/v1"
 	k8serrors "k8s.io/apimachinery/pkg/api/errors"
 	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
 	"k8s.io/apimachinery/pkg/util/httpstream"
+	"k8s.io/apimachinery/pkg/util/wait"
 	"k8s.io/cli-runtime/pkg/genericclioptions"
+	"k8s.io/client-go/informers"
 	"k8s.io/client-go/kubernetes"
 	"k8s.io/client-go/rest"
+	"k8s.io/client-go/tools/cache"
 	"k8s.io/client-go/tools/clientcmd"
 	"k8s.io/client-go/tools/portforward"
 	"k8s.io/client-go/transport/spdy"
+	resourcehelper "k8s.io/kubectl/pkg/util/resource"
+	podutil "k8s.io/kubernetes/pkg/api/v1/pod"
 
+	"github.com/apache/yunikorn-k8shim/pkg/common/utils"
 	"github.com/apache/yunikorn-k8shim/test/e2e/framework/configmanager"
 	"github.com/apache/yunikorn-k8shim/test/e2e/framework/helpers/common"
 )
@@ -140,6 +145,14 @@ func (k *KubeCtl) GetPods(namespace string) (*v1.PodList, error) {
 	return k.clientSet.CoreV1().Pods(namespace).List(context.TODO(), metav1.ListOptions{})
 }
 
+func (k *KubeCtl) GetPodsByOptions(options metav1.ListOptions) (*v1.PodList, error) {
+	return k.clientSet.CoreV1().Pods("").List(context.TODO(), options)
+}
+
+func (k *KubeCtl) GetJobs(namespace string) (*batchv1.JobList, error) {
+	return k.clientSet.BatchV1().Jobs(namespace).List(context.TODO(), metav1.ListOptions{})
+}
+
 func (k *KubeCtl) GetPod(name, namespace string) (*v1.Pod, error) {
 	return k.clientSet.CoreV1().Pods(namespace).Get(context.TODO(), name, metav1.GetOptions{})
 }
@@ -193,6 +206,18 @@ func (k *KubeCtl) GetPodNamesFromNS(namespace string) ([]string, error) {
 	return s, nil
 }
 
+func (k *KubeCtl) GetJobNamesFromNS(namespace string) ([]string, error) {
+	var s []string
+	jobs, err := k.GetJobs(namespace)
+	if err != nil {
+		return nil, err
+	}
+	for _, each := range jobs.Items {
+		s = append(s, each.Name)
+	}
+	return s, nil
+}
+
 func SetPortForwarder(req PortForwardAPodRequest, dialer httpstream.Dialer, ports []string) error {
 	lock.Lock()
 	defer lock.Unlock()
@@ -356,10 +381,45 @@ func GetConfigMapObj(yamlPath string) (*v1.ConfigMap, error) {
 	return c.(*v1.ConfigMap), err
 }
 
+func LogNamespaceInfo(ns string) error {
+	fmt.Fprintf(ginkgo.GinkgoWriter, "Log namespace info from %s\n", ns)
+	cmd := fmt.Sprintf("kubectl cluster-info dump --namespaces=%s", ns)
+	out, runErr := common.RunShellCmdForeground(cmd)
+	if runErr != nil {
+		return runErr
+	}
+	ginkgo.By("Cluster dump output:\n" + out)
+	return nil
+}
+
+func (k *KubeCtl) GetPodLogs(podName string, namespace string, containerName string) ([]byte, error) {
+	options := &v1.PodLogOptions{
+		Container: containerName,
+	}
+	logsReq := k.clientSet.CoreV1().Pods(namespace).GetLogs(podName, options)
+	logsBytes, err := logsReq.DoRaw(context.TODO())
+	if err != nil {
+		return []byte{}, err
+	}
+
+	return logsBytes, nil
+}
+
 func (k *KubeCtl) CreateConfigMap(cMap *v1.ConfigMap, namespace string) (*v1.ConfigMap, error) {
 	return k.clientSet.CoreV1().ConfigMaps(namespace).Create(context.TODO(), cMap, metav1.CreateOptions{})
 }
 
+func (k *KubeCtl) ConfigMapExists(name string, namespace string) (bool, error) {
+	_, err := k.clientSet.CoreV1().ConfigMaps(namespace).Get(context.TODO(), name, metav1.GetOptions{})
+	if err != nil {
+		if k8serrors.IsNotFound(err) {
+			return false, nil
+		}
+		return false, err
+	}
+	return true, nil
+}
+
 func (k *KubeCtl) GetConfigMap(name string, namespace string) (*v1.ConfigMap, error) {
 	return k.clientSet.CoreV1().ConfigMaps(namespace).Get(context.TODO(), name, metav1.GetOptions{})
 }
@@ -368,6 +428,21 @@ func (k *KubeCtl) UpdateConfigMap(cMap *v1.ConfigMap, namespace string) (*v1.Con
 	return k.clientSet.CoreV1().ConfigMaps(namespace).Update(context.TODO(), cMap, metav1.UpdateOptions{})
 }
 
+func (k *KubeCtl) StartConfigMapInformer(namespace string, stopChan <-chan struct{}, eventHandler cache.ResourceEventHandler) error {
+	informerFactory := informers.NewSharedInformerFactoryWithOptions(k.clientSet, 0, informers.WithNamespace(namespace))
+	informerFactory.Start(stopChan)
+	configMapInformer := informerFactory.Core().V1().ConfigMaps()
+	configMapInformer.Informer().AddEventHandler(eventHandler)
+	go configMapInformer.Informer().Run(stopChan)
+	if err := utils.WaitForCondition(func() bool {
+		return configMapInformer.Informer().HasSynced()
+	}, time.Second, 30*time.Second); err != nil {
+		return err
+	}
+
+	return nil
+}
+
 func (k *KubeCtl) DeleteConfigMap(cName string, namespace string) error {
 	return k.clientSet.CoreV1().ConfigMaps(namespace).Delete(context.TODO(), cName, metav1.DeleteOptions{})
 }
@@ -380,6 +455,22 @@ func GetPodObj(yamlPath string) (*v1.Pod, error) {
 	return o.(*v1.Pod), err
 }
 
+func (k *KubeCtl) CreateDeployment(deployment *appsv1.Deployment, namespace string) (*appsv1.Deployment, error) {
+	return k.clientSet.AppsV1().Deployments(namespace).Create(context.TODO(), deployment, metav1.CreateOptions{})
+}
+
+func (k *KubeCtl) DeleteDeployment(name, namespace string) error {
+	var secs int64 = 0
+	err := k.clientSet.AppsV1().Deployments(namespace).Delete(context.TODO(), name, metav1.DeleteOptions{
+		GracePeriodSeconds: &secs,
+	})
+	return err
+}
+
+func (k *KubeCtl) GetDeployment(name, namespace string) (*appsv1.Deployment, error) {
+	return k.clientSet.AppsV1().Deployments(namespace).Get(context.TODO(), name, metav1.GetOptions{})
+}
+
 func (k *KubeCtl) CreatePod(pod *v1.Pod, namespace string) (*v1.Pod, error) {
 	return k.clientSet.CoreV1().Pods(namespace).Create(context.TODO(), pod, metav1.CreateOptions{})
 }
@@ -407,6 +498,20 @@ func (k *KubeCtl) DeletePodGracefully(podName string, namespace string) error {
 	return err
 }
 
+func (k *KubeCtl) DeleteJob(jobName string, namespace string) error {
+	var secs int64 = 0
+	var policy = metav1.DeletePropagationForeground
+	err := k.clientSet.BatchV1().Jobs(namespace).Delete(context.TODO(), jobName, metav1.DeleteOptions{
+		GracePeriodSeconds: &secs,
+		PropagationPolicy:  &policy,
+	})
+	if err != nil {
+		return err
+	}
+	err = k.WaitForJobTerminated(namespace, jobName, 120*time.Second)
+	return err
+}
+
 // return a condition function that indicates whether the given pod is
 // currently in desired state
 func (k *KubeCtl) isPodInDesiredState(podName string, namespace string, state v1.PodPhase) wait.ConditionFunc {
@@ -415,7 +520,6 @@ func (k *KubeCtl) isPodInDesiredState(podName string, namespace string, state v1
 		if err != nil {
 			return false, err
 		}
-
 		switch pod.Status.Phase {
 		case state:
 			return true, nil
@@ -426,6 +530,19 @@ func (k *KubeCtl) isPodInDesiredState(podName string, namespace string, state v1
 	}
 }
 
+func (k *KubeCtl) isPodSelectorInNs(selector string, namespace string) wait.ConditionFunc {
+	return func() (bool, error) {
+		podList, err := k.ListPods(namespace, selector)
+		if err != nil {
+			return false, err
+		}
+		if len(podList.Items) > 0 {
+			return true, nil
+		}
+		return false, nil
+	}
+}
+
 // Return a condition function that indicates if the pod is NOT in the given namespace
 func (k *KubeCtl) isPodNotInNS(podName string, namespace string) wait.ConditionFunc {
 	return func() (bool, error) {
@@ -442,6 +559,22 @@ func (k *KubeCtl) isPodNotInNS(podName string, namespace string) wait.ConditionF
 	}
 }
 
+// Return a condition function that indicates if the job is NOT in the given namespace
+func (k *KubeCtl) isJobNotInNS(jobName string, namespace string) wait.ConditionFunc {
+	return func() (bool, error) {
+		jobNames, err := k.GetJobNamesFromNS(namespace)
+		if err != nil {
+			return false, err
+		}
+		for _, name := range jobNames {
+			if jobName == name {
+				return false, nil
+			}
+		}
+		return true, nil
+	}
+}
+
 // return a condition function that indicates whether the given pod has received the expected event
 func (k *KubeCtl) isPodEventTriggered(namespace string, podName string, expectedReason string) wait.ConditionFunc {
 	return func() (bool, error) {
@@ -459,30 +592,54 @@ func (k *KubeCtl) isPodEventTriggered(namespace string, podName string, expected
 	}
 }
 
+func (k *KubeCtl) isNumPod(namespace string, wanted int) wait.ConditionFunc {
+	return func() (bool, error) {
+		podList, err := k.GetPods(namespace)
+		if err != nil {
+			return false, err
+		}
+		if len(podList.Items) == wanted {
+			return true, nil
+		}
+		return true, nil
+	}
+}
+
+func (k *KubeCtl) WaitForJobPods(namespace string, jobName string, numPods int, timeout time.Duration) error {
+	return wait.PollImmediate(time.Millisecond*100, timeout, k.isNumJobPodsInDesiredState(jobName, namespace, numPods, v1.PodRunning))
+}
 func (k *KubeCtl) WaitForPodEvent(namespace string, podName string, expectedReason string, timeout time.Duration) error {
-	return wait.PollImmediate(time.Second, timeout, k.isPodEventTriggered(namespace, podName, expectedReason))
+	return wait.PollImmediate(time.Millisecond*100, timeout, k.isPodEventTriggered(namespace, podName, expectedReason))
 }
 
 func (k *KubeCtl) WaitForPodTerminated(namespace string, podName string, timeout time.Duration) error {
-	return wait.PollImmediate(time.Second, timeout, k.isPodNotInNS(podName, namespace))
+	return wait.PollImmediate(time.Millisecond*100, timeout, k.isPodNotInNS(podName, namespace))
+}
+
+func (k *KubeCtl) WaitForJobTerminated(namespace string, jobName string, timeout time.Duration) error {
+	return wait.PollImmediate(time.Millisecond*100, timeout, k.isJobNotInNS(jobName, namespace))
 }
 
 // Poll up to timeout seconds for pod to enter running state.
 // Returns an error if the pod never enters the running state.
 func (k *KubeCtl) WaitForPodRunning(namespace string, podName string, timeout time.Duration) error {
-	return wait.PollImmediate(time.Second, timeout, k.isPodInDesiredState(podName, namespace, v1.PodRunning))
+	return wait.PollImmediate(time.Millisecond*100, timeout, k.isPodInDesiredState(podName, namespace, v1.PodRunning))
 }
 
 func (k *KubeCtl) WaitForPodPending(namespace string, podName string, timeout time.Duration) error {
-	return wait.PollImmediate(time.Second, timeout, k.isPodInDesiredState(podName, namespace, v1.PodPending))
+	return wait.PollImmediate(time.Millisecond*100, timeout, k.isPodInDesiredState(podName, namespace, v1.PodPending))
 }
 
 func (k *KubeCtl) WaitForPodSucceeded(namespace string, podName string, timeout time.Duration) error {
-	return wait.PollImmediate(time.Second, timeout, k.isPodInDesiredState(podName, namespace, v1.PodSucceeded))
+	return wait.PollImmediate(time.Millisecond*100, timeout, k.isPodInDesiredState(podName, namespace, v1.PodSucceeded))
 }
 
 func (k *KubeCtl) WaitForPodFailed(namespace string, podName string, timeout time.Duration) error {
-	return wait.PollImmediate(time.Second, timeout, k.isPodInDesiredState(podName, namespace, v1.PodFailed))
+	return wait.PollImmediate(time.Millisecond*100, timeout, k.isPodInDesiredState(podName, namespace, v1.PodFailed))
+}
+
+func (k *KubeCtl) WaitForPodCount(namespace string, wanted int, timeout time.Duration) error {
+	return wait.PollImmediate(time.Millisecond*100, timeout, k.isNumPod(namespace, wanted))
 }
 
 // Returns the list of currently scheduled or running pods in `namespace` with the given selector
@@ -496,6 +653,17 @@ func (k *KubeCtl) ListPods(namespace string, selector string) (*v1.PodList, erro
 	return podList, nil
 }
 
+// Returns the list of currently scheduled or running pods in `namespace` with the given selector
+func (k *KubeCtl) ListPodsByFieldSelector(namespace string, selector string) (*v1.PodList, error) {
+	listOptions := metav1.ListOptions{FieldSelector: selector}
+	podList, err := k.clientSet.CoreV1().Pods(namespace).List(context.TODO(), listOptions)
+
+	if err != nil {
+		return nil, err
+	}
+	return podList, nil
+}
+
 // Wait up to timeout seconds for all pods in 'namespace' with given 'selector' to enter running state.
 // Returns an error if no pods are found or not all discovered pods enter running state.
 func (k *KubeCtl) WaitForPodBySelectorRunning(namespace string, selector string, timeout int) error {
@@ -515,6 +683,11 @@ func (k *KubeCtl) WaitForPodBySelectorRunning(namespace string, selector string,
 	return nil
 }
 
+// Wait up to timeout seconds for a pod in 'namespace' with given 'selector' to exist
+func (k *KubeCtl) WaitForPodBySelector(namespace string, selector string, timeout time.Duration) error {
+	return wait.PollImmediate(time.Millisecond*100, timeout, k.isPodSelectorInNs(selector, namespace))
+}
+
 func (k *KubeCtl) CreateSecret(secret *v1.Secret, namespace string) (*v1.Secret, error) {
 	return k.clientSet.CoreV1().Secrets(namespace).Create(context.TODO(), secret, metav1.CreateOptions{})
 }
@@ -614,7 +787,7 @@ func (k *KubeCtl) PodScheduled(podNamespace, podName string) wait.ConditionFunc
 }
 
 func (k *KubeCtl) WaitForPodScheduled(namespace string, podName string, timeout time.Duration) error {
-	return wait.PollImmediate(time.Second, timeout, k.PodScheduled(namespace, podName))
+	return wait.PollImmediate(time.Millisecond*100, timeout, k.PodScheduled(namespace, podName))
 }
 
 // PodUnschedulable returns a condition function that returns true if the given pod
@@ -674,24 +847,32 @@ func (k *KubeCtl) RemoveYunikornSchedulerPodAnnotation(annotation string) error
 	return err
 }
 
+func (k *KubeCtl) CreatePriorityClass(pc *schedulingv1.PriorityClass) (*schedulingv1.PriorityClass, error) {
+	return k.clientSet.SchedulingV1().PriorityClasses().Create(context.Background(), pc, metav1.CreateOptions{})
+}
+
+func (k *KubeCtl) DeletePriorityClass(priorityClassName string) error {
+	return k.clientSet.SchedulingV1().PriorityClasses().Delete(context.Background(), priorityClassName, metav1.DeleteOptions{})
+}
+
 func (k *KubeCtl) CreateJob(job *batchv1.Job, namespace string) (*batchv1.Job, error) {
 	return k.clientSet.BatchV1().Jobs(namespace).Create(context.TODO(), job, metav1.CreateOptions{})
 }
 
 func (k *KubeCtl) WaitForJobPodsCreated(namespace string, jobName string, numPods int, timeout time.Duration) error {
-	return wait.PollImmediate(time.Second, timeout, k.isNumJobPodsCreated(jobName, namespace, numPods))
+	return wait.PollImmediate(time.Millisecond*100, timeout, k.isNumJobPodsCreated(jobName, namespace, numPods))
 }
 
 func (k *KubeCtl) WaitForJobPodsRunning(namespace string, jobName string, numPods int, timeout time.Duration) error {
-	return wait.PollImmediate(time.Second, timeout, k.isNumJobPodsInDesiredState(jobName, namespace, numPods, v1.PodRunning))
+	return wait.PollImmediate(time.Millisecond*100, timeout, k.isNumJobPodsInDesiredState(jobName, namespace, numPods, v1.PodRunning))
 }
 
 func (k *KubeCtl) WaitForJobPodsSucceeded(namespace string, jobName string, numPods int, timeout time.Duration) error {
-	return wait.PollImmediate(time.Second, timeout, k.isNumJobPodsInDesiredState(jobName, namespace, numPods, v1.PodSucceeded))
+	return wait.PollImmediate(time.Millisecond*100, timeout, k.isNumJobPodsInDesiredState(jobName, namespace, numPods, v1.PodSucceeded))
 }
 
 func (k *KubeCtl) WaitForPlaceholders(namespace string, podPrefix string, numPods int, timeout time.Duration, podPhase v1.PodPhase) error {
-	return wait.PollImmediate(time.Second, timeout, k.isNumPlaceholdersRunning(namespace, podPrefix, numPods, podPhase))
+	return wait.PollImmediate(time.Millisecond*100, timeout, k.isNumPlaceholdersRunning(namespace, podPrefix, numPods, podPhase))
 }
 
 func (k *KubeCtl) isNumPlaceholdersRunning(namespace string, podPrefix string, num int, podPhase v1.PodPhase) wait.ConditionFunc {
@@ -768,3 +949,83 @@ func ApplyYamlWithKubectl(path, namespace string) error {
 func (k *KubeCtl) GetNodes() (*v1.NodeList, error) {
 	return k.clientSet.CoreV1().Nodes().List(context.TODO(), metav1.ListOptions{})
 }
+
+func GetWorkerNodes(nodes v1.NodeList) []v1.Node {
+	var workerNodes []v1.Node
+	for _, node := range nodes.Items {
+		scheduleable := true
+		for _, t := range node.Spec.Taints {
+			if t.Effect == v1.TaintEffectNoSchedule {
+				scheduleable = false
+				break
+			}
+		}
+		if scheduleable {
+			workerNodes = append(workerNodes, node)
+		}
+	}
+	return workerNodes
+}
+
+// Sums up current resource usage in a list of pods. Non-running pods are filtered out.
+func GetPodsTotalRequests(podList *v1.PodList) (reqs v1.ResourceList) {
+	reqs = make(v1.ResourceList)
+	for i := range podList.Items {
+		pod := podList.Items[i]
+		podReqs := v1.ResourceList{}
+		if pod.Status.Phase == v1.PodRunning {
+			podReqs, _ = resourcehelper.PodRequestsAndLimits(&pod)
+		}
+		for podReqName, podReqValue := range podReqs {
+			if value, ok := reqs[podReqName]; !ok {
+				reqs[podReqName] = podReqValue.DeepCopy()
+			} else {
+				value.Add(podReqValue)
+				reqs[podReqName] = value
+			}
+		}
+	}
+	return
+}
+
+// GetNodesAvailRes Returns map of nodeName to list of available resource (memory and cpu only) amounts.
+func (k *KubeCtl) GetNodesAvailRes(nodes v1.NodeList) map[string]v1.ResourceList {
+	// Find used resources per node
+	nodeUsage := make(map[string]v1.ResourceList)
+	for _, node := range nodes.Items {
+		nodePods, err := k.GetPodsByOptions(metav1.ListOptions{
+			FieldSelector: fmt.Sprintf("spec.nodeName=%s", node.Name),
+		})
+		if err != nil {
+			panic(err)
+		}
+		nodeUsage[node.Name] = GetPodsTotalRequests(nodePods)
+	}
+
+	// Initialize available resources as capacity and subtract used resources
+	nodeAvailRes := make(map[string]v1.ResourceList)
+	for _, node := range nodes.Items {
+		resList := node.Status.Allocatable.DeepCopy()
+		nodeAvailRes[node.Name] = resList
+
+		usage := nodeUsage[node.Name]
+		mem := resList.Memory().DeepCopy()
+		mem.Sub(usage.Memory().DeepCopy())
+		cpu := resList.Cpu().DeepCopy()
+		cpu.Sub(usage.Cpu().DeepCopy())
+		newRes := v1.ResourceList{"memory": mem, "cpu": cpu}
+		nodeAvailRes[node.Name] = newRes
+	}
+	return nodeAvailRes
+}
+
+// DescribeNode Describe Node
+func (k *KubeCtl) DescribeNode(node v1.Node) error {
+	cmd := "kubectl describe node " + node.Name
+	out, runErr := common.RunShellCmdForeground(cmd)
+	if runErr != nil {
+		return runErr
+	}
+	ginkgo.By("describe output for node is:\n" + out)
+	return nil
+}
