@@ -35,6 +35,8 @@ import (
 	"github.com/apache/yunikorn-k8shim/test/e2e/framework/configmanager"
 )
 
+const DefaultPartition = "default"
+
 type RClient struct {
 	BaseURL   *url.URL
 	UserAgent string
@@ -94,20 +96,6 @@ func (c *RClient) GetQueues(partition string) (*dao.PartitionQueueDAOInfo, error
 	return queues, err
 }
 
-func (c *RClient) GetSpecificQueueInfo(partition string, queueName string) (*dao.PartitionQueueDAOInfo, error) {
-	queues, err := c.GetQueues(partition)
-	if err != nil {
-		return nil, err
-	}
-	var allSubQueues = queues.Children
-	for _, subQ := range allSubQueues {
-		if subQ.QueueName == queueName {
-			return &subQ, nil
-		}
-	}
-	return nil, fmt.Errorf("QueueInfo not found: %s", queueName)
-}
-
 func (c *RClient) GetHealthCheck() (dao.SchedulerHealthDAOInfo, error) {
 	req, err := c.newRequest("GET", configmanager.HealthCheckPath, nil)
 	if err != nil {
@@ -118,12 +106,27 @@ func (c *RClient) GetHealthCheck() (dao.SchedulerHealthDAOInfo, error) {
 	return healthCheck, err
 }
 
-func (c *RClient) GetApps(partition string, queueName string) ([]map[string]interface{}, error) {
+func (c *RClient) GetAllAppInfos() (*dao.ApplicationsDAOInfo, error) {
+	appsInfos := new(dao.ApplicationsDAOInfo)
+
+	req, err := c.newRequest("GET", configmanager.AppsPath, nil)
+	if err != nil {
+		return nil, err
+	}
+
+	_, err = c.do(req, &(appsInfos.Applications))
+	if err != nil {
+		return nil, err
+	}
+	return appsInfos, nil
+}
+
+func (c *RClient) GetApps(partition string, queueName string) ([]*dao.ApplicationDAOInfo, error) {
 	req, err := c.newRequest("GET", fmt.Sprintf(configmanager.AppsPath, partition, queueName), nil)
 	if err != nil {
 		return nil, err
 	}
-	var apps []map[string]interface{}
+	var apps []*dao.ApplicationDAOInfo
 	_, err = c.do(req, &apps)
 	return apps, err
 }
@@ -174,13 +177,12 @@ func (c *RClient) WaitForAllocationLog(partition string, queueName string, appID
 	return nil
 }
 
-func (c *RClient) isAppInDesiredState(partition string, queueName string, appID string, state string) wait.ConditionFunc {
+func (c *RClient) isAppInDesiredState(partition string, queue string, appID string, state string) wait.ConditionFunc {
 	return func() (bool, error) {
-		appInfo, err := c.GetAppInfo(partition, queueName, appID)
+		appInfo, err := c.GetAppInfo(partition, queue, appID)
 		if err != nil {
 			return false, nil // returning nil here for wait & loop
 		}
-
 		switch appInfo.State {
 		case state:
 			return true, nil
@@ -191,8 +193,18 @@ func (c *RClient) isAppInDesiredState(partition string, queueName string, appID
 	}
 }
 
-func (c *RClient) WaitForAppStateTransition(partition string, queueName string, appID string, state string, timeout int) error {
-	return wait.PollImmediate(time.Second, time.Duration(timeout)*time.Second, c.isAppInDesiredState(partition, queueName, appID, state))
+func (c *RClient) GetNodes(partition string) (*[]dao.NodesDAOInfo, error) {
+	req, err := c.newRequest("GET", fmt.Sprintf(configmanager.NodesPath, partition), nil)
+	if err != nil {
+		return nil, err
+	}
+	var nodes []dao.NodesDAOInfo
+	_, err = c.do(req, &nodes)
+	return &nodes, err
+}
+
+func (c *RClient) WaitForAppStateTransition(partition string, queue string, appID string, state string, timeout int) error {
+	return wait.PollImmediate(time.Millisecond*300, time.Duration(timeout)*time.Second, c.isAppInDesiredState(partition, queue, appID, state))
 }
 
 func (c *RClient) AreAllExecPodsAllotted(partition string, queueName string, appID string, execPodCount int) wait.ConditionFunc {
@@ -224,7 +236,7 @@ func (c *RClient) ValidateSchedulerConfig(cm v1.ConfigMap) (*dao.ValidateConfRes
 func isRootSched(policy string) wait.ConditionFunc {
 	return func() (bool, error) {
 		restClient := RClient{}
-		qInfo, err := restClient.GetQueues("default")
+		qInfo, err := restClient.GetQueues(DefaultPartition)
 		if err != nil {
 			return false, err
 		}
@@ -232,7 +244,7 @@ func isRootSched(policy string) wait.ConditionFunc {
 			return false, errors.New("no response from rest client")
 		}
 
-		if policy == "default" {
+		if policy == DefaultPartition {
 			return len(qInfo.Properties) == 0, nil
 		} else if qInfo.Properties["application.sort.policy"] == policy {
 			return true, nil
@@ -286,7 +298,7 @@ func (c *RClient) GetQueue(partition string, queueName string) (*dao.PartitionQu
 func compareQueueTS(queuePathStr string, ts string) wait.ConditionFunc {
 	return func() (bool, error) {
 		restClient := RClient{}
-		qInfo, err := restClient.GetQueue("default", "root")
+		qInfo, err := restClient.GetQueue(DefaultPartition, "root")
 		if err != nil {
 			return false, err
 		}
@@ -307,3 +319,52 @@ func AllocLogToStrings(log []dao.AllocationAskLogDAOInfo) []string {
 	}
 	return result
 }
+
+func (c *RClient) LogAppsInfo(ns string) error {
+	appsInfo, getAppErr := c.GetApps(DefaultPartition, "root."+ns)
+	if getAppErr != nil {
+		return getAppErr
+	}
+	appJSON, appJSONErr := json.MarshalIndent(appsInfo, "", "    ")
+	if appJSONErr != nil {
+		return appJSONErr
+	}
+	By("Apps REST API response is\n" + string(appJSON))
+	return nil
+}
+
+func (c *RClient) LogQueuesInfo() error {
+	qInfo, getQErr := c.GetPartitions(DefaultPartition)
+	if getQErr != nil {
+		return getQErr
+	}
+	qJSON, qJSONErr := json.MarshalIndent(qInfo, "", "    ")
+	if qJSONErr != nil {
+		return getQErr
+	}
+	By("Queues REST API response is\n" + string(qJSON))
+	return nil
+}
+
+func (c *RClient) LogNodesInfo() error {
+	nodesInfo, getNodeErr := c.GetNodes(DefaultPartition)
+	if getNodeErr != nil {
+		return getNodeErr
+	}
+	nodesJSON, nodeJSONErr := json.MarshalIndent(nodesInfo, "", "    ")
+	if nodeJSONErr != nil {
+		return nodeJSONErr
+	}
+	By("Node REST API response is\n" + string(nodesJSON))
+	return nil
+}
+
+func (c *RClient) GetPartitions(partition string) (*dao.PartitionQueueDAOInfo, error) {
+	req, err := c.newRequest("GET", fmt.Sprintf(configmanager.QueuesPath, partition), nil)
+	if err != nil {
+		return nil, err
+	}
+	var partitions *dao.PartitionQueueDAOInfo
+	_, err = c.do(req, &partitions)
+	return partitions, err
+}
