@@ -19,6 +19,7 @@
 package predicates_test
 
 import (
+	"context"
 	"fmt"
 	"reflect"
 	"time"
@@ -31,6 +32,7 @@ import (
 	"k8s.io/kubernetes/test/e2e/framework"
 	e2enode "k8s.io/kubernetes/test/e2e/framework/node"
 
+	tests "github.com/apache/yunikorn-k8shim/test/e2e"
 	"github.com/apache/yunikorn-k8shim/test/e2e/framework/configmanager"
 	"github.com/apache/yunikorn-k8shim/test/e2e/framework/helpers/common"
 	"github.com/apache/yunikorn-k8shim/test/e2e/framework/helpers/k8s"
@@ -110,11 +112,19 @@ var _ = Describe("Predicates", func() {
 
 	AfterEach(func() {
 		By("Cleanup")
+		testDescription := ginkgo.CurrentGinkgoTestDescription()
 		for _, n := range []string{ns, anotherNS} {
+			testDescription = ginkgo.CurrentGinkgoTestDescription()
+			if testDescription.Failed {
+				tests.LogTestClusterInfoWrapper(testDescription.TestText, []string{n})
+			}
 			ginkgo.By("Tear down namespace: " + n)
 			err = kClient.TearDownNamespace(n)
 			Ω(err).NotTo(HaveOccurred())
 		}
+		if testDescription.Failed {
+			tests.LogYunikornContainer(testDescription.TestText)
+		}
 	})
 
 	// Test Nodes does not have any label, hence it should be impossible to schedule Pod with
@@ -318,11 +328,11 @@ var _ = Describe("Predicates", func() {
 			Value:  "testing-taint-value",
 			Effect: v1.TaintEffectNoSchedule,
 		}
-		err = controller.AddOrUpdateTaintOnNode(kClient.GetClient(), nodeName, testTaint)
+		err = controller.AddOrUpdateTaintOnNode(context.Background(), kClient.GetClient(), nodeName, testTaint)
 		Ω(err).NotTo(HaveOccurred())
 		framework.ExpectNodeHasTaint(kClient.GetClient(), nodeName, testTaint)
 		defer func(c kubernetes.Interface, nodeName string, taint *v1.Taint) {
-			err = controller.RemoveTaintOffNode(c, nodeName, nil, taint)
+			err = controller.RemoveTaintOffNode(context.Background(), c, nodeName, nil, taint)
 			Ω(err).NotTo(HaveOccurred())
 		}(kClient.GetClient(), nodeName, testTaint)
 
@@ -365,12 +375,12 @@ var _ = Describe("Predicates", func() {
 			Value:  "testing-taint-value",
 			Effect: v1.TaintEffectNoSchedule,
 		}
-		err = controller.AddOrUpdateTaintOnNode(kClient.GetClient(), nodeName, testTaint)
+		err = controller.AddOrUpdateTaintOnNode(context.Background(), kClient.GetClient(), nodeName, testTaint)
 		Ω(err).NotTo(HaveOccurred())
 
 		framework.ExpectNodeHasTaint(kClient.GetClient(), nodeName, testTaint)
 		defer func(c kubernetes.Interface, nodeName string, taint *v1.Taint) {
-			err = controller.RemoveTaintOffNode(c, nodeName, nil, taint)
+			err = controller.RemoveTaintOffNode(context.Background(), c, nodeName, nil, taint)
 			Ω(err).NotTo(HaveOccurred())
 		}(kClient.GetClient(), nodeName, testTaint)
 
@@ -413,7 +423,7 @@ var _ = Describe("Predicates", func() {
 		Ω(logEntries).To(ContainElement(MatchRegexp(".*taint.*")), "Log entry message mismatch")
 
 		// Remove taint off the node and verify the pod is scheduled on node.
-		err = controller.RemoveTaintOffNode(kClient.GetClient(), nodeName, nil, testTaint)
+		err = controller.RemoveTaintOffNode(context.Background(), kClient.GetClient(), nodeName, nil, testTaint)
 		Ω(err).NotTo(HaveOccurred())
 		Ω(kClient.WaitForPodRunning(ns, podNameNoTolerations, time.Duration(60)*time.Second)).NotTo(HaveOccurred())
 
